/*
 ---------------------------------------------------------------
 Copyright (c) 2006, ixi software.
 This work is licensed under a Creative Commons 
 Attribution-NonCommercial-ShareAlike 2.0 England & Wales License.
 http://creativecommons.org/licenses/by-nc-sa/2.0/uk/
 --------------------------------------------------------------- 
 */

/*ok, so far we have been able to draw a circle that bounces against the window
 limits changging random color when colliding. But what happens if we want to create
 two or more circles doing the same behavior?
 we would need to declare variables like x,y,deltax,deltay for each of them. Some thing like this:
 float x = 1;
 float y = random(50,150);
 float deltaX = random(1,5);
 float deltaY = random(1,5);
 float x2 = 1;
 float y2 = random(50,150);
 float deltaX2 = random(1,5);
 float deltaY2 = random(1,5);
  ... and so on ...
  and check for collision each of them in each draw().
  This would look pretty ugly and difficult to understand if we have, say 5 circles.
  We can in this case define a class that describe the behavior of a circle, 
  and since they are going to be similar, instanciate it as many times as we need.
  In this case we will create two instances of the class.
  note that using classes is not always needed, there are programming languages
  that do not use OOP, it depends very much on what you need to solve, diferent problems
  requiere different solutions and OOP despite of being very handy and powerful is not 
  always THE solution.
 */

/* note that both circles change color when one of them bounces with limits
 this is because when you call fill() all subsequent drawing operations will
 be done with the color set by fill() function. Same happens with noStroke() 
 and some other functions. In order for each of the circles to have an independent
 color from the other we would need to remember the color for each of them 
 and set the right color everytime we draw one of them, we will do this in next
 example
 */

// declare two variables to hold instances of CircleBounce class
CircleBounce c, d;

void setup() { // just when starting up
  frameRate(20); // fps
  size(300, 200); // window size
  background(100); // bg color
  noStroke();
  changeColor();
  c = new CircleBounce(); // instanciating the class into the variables
  d = new CircleBounce();
}


color randRGB(){ // returns data type color
  return color(random(255), random(255), random(255));
}

void changeColor(){
  color c1 = randRGB();
  fill(c1);
}


void draw(){ // this is our main loop
  background(100);
  c.circleBounceDraw(); // call the draw funtion in each instance
  d.circleBounceDraw();
}


class CircleBounce{
  // this are the variable used by instances of this class
  float x = 1;
  float y = random(50,150);
  float deltaX = random(1,5);
  float deltaY = random(1,5);

  void circleBounceDraw(){ // this function could be call whatever.
    ellipse(x, y, 55, 55);
    x = x+deltaX; // moves to the right by deltaX amouth
    y = y+deltaY; // moves to the right by deltaX amouth
    if (x>=300 || x<=0){
      deltaX = -deltaX; //change X direction
      changeColor();
    }
    if (y>=200 || y<=0){
      deltaY = -deltaY; //change Y direction
      changeColor();
    }
  }
}


